-- enemy.lua
-- an enemy has several eyes and moves sporadically

Enemy = Class(function(self, g, p, str, range, speed, sight)
	self.g = g -- game to which the enemy belongs
	self.p = p
	self.v = vector(0,0)
	self.range = range or 150
	self.speed = speed or 100
	self.sight = sight or 240 -- how far can it see
	self.drag = 2
	self.eyes = {}
	for i=1,str do
		table.insert(self.eyes, Eye(g, vector(math.random()*4,math.random())*4, str+4))
	end
	self.size = str*5
end)

function Enemy:update(dt)
-- update
	for i,eye in ipairs(self.eyes) do
		-- eyes are repelled from each other when on the player
		-- but never leave the body
		-- calculate the total eye repulsion forces
		local force = vector(0,0)
		for j,eye2 in ipairs(self.eyes) do
			if eye ~= eye2 then
				local d = eye2.p-eye.p
				local dlen = d:len()
				if dlen < (eye.size+eye2.size) then
					force = force - (d:normalized())/dlen*5
				else
					force = force - (d:normalized())/dlen
				end
			end
		end
		force = force*100
		
		-- we never want to force the eye out of the body
		-- so if the position is greater than 
		--if eye.p:len2() > math.pow(self.size-7, 2) then
			force = force - eye.p*eye.p:len()/8
		--end
		if force:len2() > 1000 then
			force = force/force:len2() * 1000
		end
		eye.p = eye.p + force*dt
		eye.force = force
		lastForce = force
		eye.p:rotate_inplace(0.4*dt*eye.p:len()/self.size)
		
		-- also if any eye is within range of the player's eye, shoot it
		--if we have a target already and it's in range, keep shooting it
		if eye.shooting and 
			((eye.p+self.p):dist(eye.shooting.p+g.p.p) > self.range or
			eye.shooting.health < 0) then
			eye.shooting = nil
		end
		if eye.shooting == nil then
			-- find the nearest target and shoot it
			local minD = 100000
			local target = nil
			for j,eye2 in ipairs(g.p.eyes) do
				d = (eye.p+self.p):dist(eye2.p+g.p.p)
				if d < minD then
					minD = d
					target = eye2
				end
			end
			eye.minD = minD
			if target and (minD < self.range) then
				eye.shooting = target
			end
		end
		
		eye:update(dt)
	end
	
	-- apply drag forces
	-- self.v = self.v-self.v*self.drag*dt
	
	-- if love.mouse.isDown('l') then
		-- move towards the mouse
		-- local dmouse = self.g.c:mousepos()-self.p
		-- if dmouse:len() > self.size then
		-- 	self.v = dmouse:normalized()*self.speed
		-- end
	-- end
	local f = self.g.p.p - self.p
	if f:len() < (self.range+self.size +self.g.p.size+3) or f:len() > self.sight then
		f = vector(0,0)
	end
	f = f*2 - self.v*self.drag
	self.v = self.v + f*dt
	if self.v:len() > self.speed then
		self.v = self.v:normalized()*self.speed
	end
	self.p = self.p + self.v*dt
	self.size = 20+math.sqrt(#(self.eyes)/2)
end


function Enemy:draw()
	-- draw the boundaries
	love.graphics.setColor(200,220,220,255)
	love.graphics.circle("line", self.p.x, self.p.y, self.sight, 128)
	love.graphics.setColor(255,220,220,255)
	love.graphics.circle("line", self.p.x, self.p.y, self.range, 64)
	-- draw the body
	love.graphics.setColor(255,255,255,255)
	love.graphics.circle("fill", self.p.x, self.p.y, self.size, 32)
	drawCircle(self.p,self.size)
	-- draw the eyes
	for i,eye in ipairs(self.eyes) do
		local p = self.p+eye.p
		if self.p:dist(self.g.p.p) > self.sight then
			eye:draw(p, vector(-400,-300))
		else
			eye:draw(p, self.g.p.p)
		end
		--drawCircle(p+eye.force, 2)
	end
	for i,eye in ipairs(self.eyes) do
		local p = self.p+eye.p
		if eye.shooting then
			p2 = eye.shooting.p
			love.graphics.setColor(255,0,0,255)
			love.graphics.line(p.x, p.y, (p2+g.p.p).x,(p2+g.p.p).y)
		end
	end
end