-- game.lua
-- the game class can have several states
-- using different draw, update functions

require 'Player.lua'
require 'Enemy.lua'

Game = Class(function(self)
	
	self.c = Camera(vector(0,0), 1, 0)
	self.p = Player(self)
	
	love.graphics.setBackgroundColor(255,255,255)
	-- scatter some eyes
	self.eyes = {}
	for i=1,1 do
		table.insert(self.eyes, Eye(self,vector(math.random()*800-400,math.random()*600-300), math.random()*4+4))
	end
	
	self.e = Enemy(self, vector(200,200), 4)
end)

function Game:update(dt)
	dt = math.min(dt, 0.1) -- min 10 fps
	self.p:update(dt)
	for i,eye in ipairs(self.eyes) do
		if eye.p:dist(self.p.p) < (self.p.size+eye.size) then
			eye.p = vector(0,0) --set the eye position to zero and then
			table.insert(self.p.eyes, eye) -- insert the eye to teh player
			table.remove(self.eyes, i) -- remove from the list of eyes
			love.audio.play(Eye.blinkSound)
		end
	end
	if #self.eyes == 0 then
		for i=1,1 do
			table.insert(self.eyes, Eye(self,vector(math.random()*800-400,math.random()*600-300), math.random()*4+4))
		end
	end
	
	self.e:update(dt)
	
end

function Game:draw()
	self.c:predraw()
	
	love.graphics.setColor(0,0,0,255)
	love.graphics.print('Click to move.',-380,-260)
	
	for i,eye in ipairs(self.eyes) do
		eye:draw()
	end
	
	self.e:draw()
	
	self.p:draw()
	
	self.c:postdraw()
end

function Game:keyPressed(k)
	self.p:keyPressed(k)
end