-- player.lua
require 'Eye.lua'

Player = Class(function(self, g)
	self.g = g -- game object this player belongs to
	self.p = vector(0,0) -- position
	self.v = vector(0,0) -- velocity
	self.speed = 200 -- top moving speed
	self.drag = 6
	self.eyes = {Eye(g,vector(math.random()*10,math.random()*10))}
	self.size = 20
end)

function Player:update(dt)
	-- update
	for i,eye in ipairs(self.eyes) do
		if eye.health < 0 then
			table.remove(self.eyes, i)
		end
		-- eyes are repelled from each other when on the player
		-- but never leave the body
		-- calculate the total eye repulsion forces
		local force = vector(0,0)
		for j,eye2 in ipairs(self.eyes) do
			if eye ~= eye2 then
				local d = eye2.p-eye.p
				local dlen = d:len()
				if dlen < (eye.size+eye2.size) then
					force = force - (d:normalized())/dlen*5
				else
					force = force - (d:normalized())/dlen
				end
			end
		end
		force = force*100
		
		-- we never want to force the eye out of the body
		-- so if the position is greater than 
		--if eye.p:len2() > math.pow(self.size-7, 2) then
			force = force - eye.p*eye.p:len()/8
		--end
		if force:len2() > 1000 then
			force = force/force:len2() * 1000
		end
		eye.p = eye.p + force*dt
		eye.force = force
		lastForce = force
		eye.p:rotate_inplace(0.4*dt*eye.p:len()/self.size)
		eye:update(dt)
	end
	
	-- apply drag forces
	self.v = self.v-self.v*self.drag*dt
	
	if love.mouse.isDown('l') then
		-- move towards the mouse
		local dmouse = self.g.c:mousepos()-self.p
		if dmouse:len() > self.size then
			self.v = dmouse:normalized()*self.speed
		end
	end
	
	self.p = self.p + self.v*dt
	self.size = 20+math.sqrt(#(self.eyes)/2)
end

function Player:draw()
	-- draw the body
	love.graphics.setColor(255,255,255,255)
	love.graphics.circle("fill", self.p.x, self.p.y, self.size, 32)
	drawCircle(self.p,self.size)
	-- draw the eyes
	for i,eye in ipairs(self.eyes) do
		local p = self.p+eye.p
		eye:draw(p)
		--drawCircle(p+eye.force, 2)
	end
end

function drawHair(p,dir,len)
	-- draw a hair
end

function Player:keyPressed(k)
	if k == 'i' then
		--table.insert(self.eyes, Eye(vector(0,0)));
		--love.audio.play(Eye.blinkSound)
	end
end